@@ -6,8 +6,7 @@
 (defgroup greader-mac
   nil
   "Back-end of mac for greader."
-  :group 'greader
-  )
+  :group 'greader)
 
 (defcustom greader-mac-voice nil
   "Set the desired voice for the tts `say’.
@@ -71,6 +70,8 @@ COMMAND must be a string suitable for `make-process'."
        (greader-mac-set-rate arg))))
     ('punctuation
      nil)
+    ('get-rate
+     greader-mac-rate)
     (_
      'not-implemented)))
 (put 'greader-mac 'greader-backend-name "greader-mac")
